# 91isoft_spbt

## 分支

+ v3.2.0_WF_A 
  + workflow 选型 activiti
+ v3.2.0_WF_F
    + workflow 选型 flowable
+ v3.2.0_DB_shard
    + 分库 shardingsphere-jdbc

## 版本管理

| 版本   | 维护人员  | 维护时间       | 主要内容                             | 
|------| ------------ |------------|----------------------------------|
| v1.0 | 李丛阳  | 2018-12-14 | 基础结构搭建                           |
| v2.0 | 李丛阳  | 2019-07-12 | springboot2.0.1升级                |
| v2.3 | 李丛阳  | 2021-04-01 | 多数据库，目录划分                        |
| v3.0 | 李丛阳  | 2022-04-16 | jdk17升级，集成cxf，CICD               |
| v3.1 | 李丛阳  | 2022-08-25 | mongodb支持，事务                     |
| v3.2 | 李丛阳  | 2024-01-05 | 代码生成器升级，数据库适配升级，aj-captcha，脱敏序列化 |

## 项目组件清单


| 版本   | 组件                 | 描述                                                       | 维护人 |   
|------|--------------------|----------------------------------------------------------|------------ |
| v1.0 | lombok             | 简化冗余代码，连通编辑器和构建工具                                        | cy |
| v1.0 | ssm                | springmvc+spring+mybatis+mybatis扩展                       | cy |
| v1.0 | springboot         | 简化spring配置                                               | cy |
| v1.0 | springfox-swagger2 | 在线api                                                    | cy |
| v1.0 | logback            | 日志                                                       | cy |
| v1.0 | jwt                | json web token 身份验证                                      | cy |
| v1.0 | spring-security    | 安全性框架                                                    | cy |
| v2.0 | springboot         | springboot2.0升级                                          | cy |
| v2.0 | activiti           | 工作流框架                                                    | cy |
| v2.1 | knife4j            | swagger升级                                                | cy |
| v2.1 | hikariCP           | 代替druid                                                  | cy |
| v2.2 | 安全性防御              | xss sm4的xss csrf 表单重复提交                                  | cy |
| v2.2 | 登录日志/操作日志          | @CyLog                                                   | cy |
| v2.3 | 权限                 | 支持wbac模式和rbac模式                                          | cy |
| v2.3 | 数据库扩展              | 支持mysql5.7 oracle12c sqlserver2019                       | cy |
| v3.0 | webservice         | 使用cxf 发布webservice，测试调用                                  | cy |
| v3.0 | devops             | 支持kubeSphere CICD                                        | cy |
| v3.0 | jdk 17.0           | 接入jdk 17                                                 | cy |
| v3.1 | mongodb4.2.x       | 事务(需要服务自身支持)                                             | cy |
| v3.2 | rdbms              | 适配升级mysql8.0.30；oracle19c；postgreSQL9.6.24；sqlserver2019 | cy |
| v3.2 | 代码生成器              | 增加前端代码生成 .js .vue                                        | cy |
| v3.2 | aj-captcha         | 增加滑块，中文点选                                                | cy |

## 设计规范

### 数据库设计规范

| 编号  | 类型 | 标题  | 描述  |
| ------------ | ------------ | ------------ | ------------ |
| 1  | 表  | 表名  | 小写，单词之间使用下划线  |
| 2  | 表  | 表名  | 中间表使用r_间隔 eg: b_r_teacher_course  |
| 3  | 表  | 表结构  | 主表必须要有6+字段(创建时间，修改时间等)，中间表不需要  |
| 4  | 列  | 列名  | 小写，单词之间使用下划线  |
| 5  | 列  | 列名  | 每一列必须要有备注  |
| 6  | 列  | 状态列  | char(1) 或 varchar(1)，备注要描述清楚状态  |

### 微服务编码规范

| 编号  | 类型 | 标题  | 描述  |
| ------------ | ------------ | ------------ | ------------ |
| 0  | 结构  | 目录规范  | 详见 91isoft_spbt_project|
| 1  | 接口  | 提供其它服务调用的接口要标明  | 模块要单独提出一个controller,requestMappeing使用/api/ 开头|
| 2  | 接口  | 返回值  | Result / GridModel|
| 3  | 接口   |  使用restful风格 | get post pust delete，/id/ |
| 4  | 接口   |  requestMappeing命名规范 | queryUsers, queryUserByPagination, detail/businessId, delete/businessId, add, update/businessId    |
| 5  | 接口   |  swagger | controller api docs |
| 6  | 结构   |  service dao | 注释，尽量复用 |
| 7  | 结构   |  tcc | 分布式事务书写规范 |
| 8  | 结构   |  entity | @Entity @Data @NoArgsConstructor @AllArgsConstructor @Table |

### sql规范

~~~~
#!bin/bash
${basedir}/sql
            /mysql
                /init
                /activiti
                /bak
            /oracle
                /init
                /activiti
                /bak
            /postgres
                /init
                /activiti
                /bak
            /sqlserver
                /init
                /activiti
                /bak           
           
~~~~

## 插件

### path

+ swagger `http://localhost:8084/swagger-ui.html#/`
+ knife `http://localhost:8084/doc.html#/`

### easypoi

`https://opensource.afterturn.cn/doc/easypoi.html#201`

### 打包

+ `mvn package -DskipTests`
+ `mvn deploy -DskipTests` 


### sonar

~~~~
<dependency>
    <groupId>org.codehaus.sonar</groupId>
    <artifactId>sonar-maven-plugin</artifactId>
    <version>5.1</version>
</dependency>

mvn sonar:sonar -Dsonar.host.url=http://sonarqube.91isoft.com:10102
~~~~

`http://localhost:8082/static/explorer/modeler.html?modelId=ee5e3bc34bc041d293cc5e2e0363d184`

### webservice

下载cxf，配置环境变量
```shell
//-p 包名 -d 目录
wsdl2java -p com -d D:\\src -all xx.wsdl

wsdl2java -d . -client http://192.168.1.12:8082/cxf/workServer?wsdl

```

> `wsdl2java` 工具，反转的时候，可能会出现属性大小写问题。建议可以采用 intellij Idea， 通过增加 `webservice client`模块，提示安装 `soap UI`插件; 
> 重启后 tools -> soap UI -> generate 构建wsdl代码


## 信息安全

### RSA参数加密

非对称加密，前端整体加密，参数只发送 sign 参数
```java
public CyResult login(@RequestParam("sign") String sign){
    CyRSAUtil.decryptByPrivateKey(null,sign);
    loginInfo=JSONUtil.toBean(userJson,CyLoginInfo.class);
    username=loginInfo.getUsername();
    password=loginInfo.getPassword();
    ...
}    
```


### 参数加签

按字段顺序加签sign，使用md5，前端 源参数和sign一起发送服务端，防止篡改
```java
// 1. 开启加签
cy:
  model:
    decryptParam: true
    decryptKey: cyKey

//作用域
@Around("@annotation(org.rcisoft.core.anno.CyEncryptSignAnno)")

// 2.排顺序，重新加密，比较
StringBuilder str = new StringBuilder();
for (Map.Entry e :
  param.entrySet()) {
  str.append(e.getKey());
  str.append("=");
  str.append(e.getValue());
  str.append("&");
}
str.append("key=" + cyGlobal.getDecryptKey());
//加密并大写
String decrypt = this.md5Encode(str.toString(), "utf-8").toUpperCase();
//比较加密的数据与前台发来的sign是否一致
if (!decrypt.equals(request.getParameter("sign"))) {
  throw new CyServiceException(CyResSysExcEnum.PARAMETER_DECRYPT_ERROR);
}
```

### SM4参数加密


sm4 加密
```java
// 1. 开启加签
cy:
  model:
    decryptSm4Param: true
    decryptSm4Secret: 'FFFAAA333777EEEB'

// 2. 作用域
@Around("@annotation(org.rcisoft.core.anno.CyEncryptSm4Anno)")
@Around("execution(* org.rcisoft..*.controller.*Controller.*(..))") 
        
        
// 2.解密，赋值到第一个不是 string || integer 形参上
String content = cySM4Util.decryptData_ECB(sign);
args[0] = JSONUtil.toBean(content, args[0].getClass());
this.xssDefense(args[0], saveHtml, isGet);

//3. 跳过 xss
@CySm4ExcludeXssAnno

//4. 跳过sm4
@CySm4ExcludedAnno
```



### 配置文件加密

* 将需要加密的数据通过工具类进行加密
> 例：对数据库密码进行加密，先使用`org.rcisoft.core.util.CyENCUtil`工具类的main函数，对数据库的密码进行明文加密，其中`salt`是加密盐值，
> `PBEWITHHMACSHA512ANDAES_128`是加密算法，需要和代码中的加密数据相同。

* 将生成的加密字符串用`ENC()`包装放入配置文件即可
> 例：password: ENC(n8ZThnsHi7lFa56+3uHL3zX6kunkhj1DZXC7g3RLz/t48AJTmQurmhuVeJ0MOZRg)
> 其中的“n8ZThnsHi7lFa56+3uHL3zX6kunkhj1DZXC7g3RLz/t48AJTmQurmhuVeJ0MOZRg”是加密密文

```yaml
# sample
cy_db:
  ip: ENC(vw6fzLP+x...vUfIWgqsYrzKRNKTfxLp)
  port: ENC(HebzQAlq/De7b7Nt...qkarc9LzIT5WH49UGfpa)
  username: ENC(oBZSjAdFD...fvnS2aHy)
  password: ENC(v2dv818F45rL4...T9j4aDccy7FND4QHN1gyDZqG3cY8w8Q==)
  db: exam
```

### 参数校验

基础参数格式，后端校验

```java
//1. 缺省开启
cy:
  model:
    entityParam: true
// 作用域
@Around("execution(* org.rcisoft..*.controller.*Controller.update*(..))")
@Around("execution(* org.rcisoft..*.controller.*Controller.add*(..))")

//2. 使用，注意 参数顺序
public CyResult update(@Valid SysUserRbac sysUser, BindingResult bindingResult) {
        ...
}

@NotNull
private String username;
```

### xss防御

自动开启，无法关闭
```java

//1. sm4 排除 xss
@CySm4ExcludeXssAnno

//2. 其他排除
cy:
  model:
    xss:
      excludes:
        - "/static/*"
        - "/swagger-resources/*"
        - "/api-docs/*"
        - "/webjars/*"

```


### 数据脱敏

```java
// 目标支持 EMAIL MOBILE_PHONE ID_CARD
@CySensitiveAnno(value = CySensitiveEnum.EMAIL)
```





